import "package:benchmark_harness/benchmark_harness.dart";
import "package:poker/poker.dart";

class ExhaustiveBenchmark extends BenchmarkBase {
  ExhaustiveBenchmark() : super('Exhaustive');

  late Evaluator evaluator;

  late EvaluationResult result;

  // The benchmark code.
  @override
  void run() {
    for (final r in evaluator) {
      result += r;
    }
  }

  @override
  void setup() {
    evaluator = ExhaustiveEvaluator(
      communityCards: CardSet.fromString("3c6dTs4d5d"),
      players: [
        HandRange.fromString("As3h"),
        HandRange.fromString("8d8h"),
        HandRange.fromString("AQs-ATsAKo-AJo44+"),
      ],
    );
    result = EvaluationResult.empty(playerLength: 3);

    print(evaluator.length);
    print(evaluator.fold<int>(0, (l, _) => l + 1));
  }

  @override
  void teardown() {
    print(result);
  }
}

class MontecarloBenchmark extends BenchmarkBase {
  MontecarloBenchmark() : super('Montecarlo');

  late Evaluator evaluator;

  late EvaluationResult result;

  // The benchmark code.
  @override
  void run() {
    for (final r in evaluator.take(1000000)) {
      result += r;
    }
  }

  @override
  void setup() {
    evaluator = MontecarloEvaluator(
      communityCards: CardSet.fromString("3c6dTs4d"),
      players: [
        HandRange.fromString("As3h"),
        HandRange.fromString("8d8h"),
        HandRange.fromString("AQs-ATsAKo-AJo44+"),
      ],
    );
    result = EvaluationResult.empty(playerLength: 3);
  }

  @override
  void teardown() {
    print(result);
  }
}

void main() {
  ExhaustiveBenchmark().report();
  MontecarloBenchmark().report();
}
