import "dart:collection";
import "dart:math";
import "package:poker/poker.dart";
import "./evaluator.dart";

class MontecarloEvaluator
    with IterableMixin<EvaluationResult>
    implements Evaluator {
  MontecarloEvaluator({
    required this.communityCards,
    required this.players,
  });

  final CardSet communityCards;

  final List<HandRange> players;

  Iterator<EvaluationResult> get iterator =>
      MontecarloEvaluationIterator(original: this);

  @override
  int get length => -1;
}

class MontecarloEvaluationIterator implements Iterator<EvaluationResult> {
  MontecarloEvaluationIterator({required MontecarloEvaluator original})
      : _playerCardPairs = original.players.map((p) => p.toList()).toList(),
        _baseDeck = CardSet.full.difference(original.communityCards),
        _baseCommunityCards = original.communityCards,
        _lastResult = null;

  final List<List<CardPair>> _playerCardPairs;

  final CardSet _baseDeck;

  final CardSet _baseCommunityCards;

  EvaluationResult? _lastResult;

  @override
  EvaluationResult get current => _lastResult!;

  @override
  bool moveNext() {
    final chosenPlayerCardPairs = _playerCardPairs
        .map((cps) => cps[Random().nextInt(cps.length)])
        .toList();
    CardSet deck = _baseDeck;
    CardSet communityCards = _baseCommunityCards;
    bool isPossible = true;

    for (final cardPair in chosenPlayerCardPairs) {
      if (!deck.contains(cardPair.first) || !deck.contains(cardPair.last)) {
        isPossible = false;

        break;
      }

      deck = deck.difference(cardPair);
    }

    if (isPossible) {
      for (int i = communityCards.length; i < 5;) {
        final card = Card.fromInt(pow(2, Random().nextInt(52)).toInt());

        if (deck.contains(card)) {
          deck = deck.difference(CardSet.single(card));
          communityCards = communityCards.union(CardSet.single(card));

          i += 1;
        }
      }

      final matchup = Matchup.showdown(
        communityCards: communityCards,
        playerCardPairs: chosenPlayerCardPairs,
      );

      _lastResult = EvaluationResult.fromMatchup(matchup);
    } else {
      _lastResult =
          EvaluationResult.empty(playerLength: chosenPlayerCardPairs.length);
    }

    return true;
  }
}
