import "package:poker/poker.dart";

class EvaluationResult {
  EvaluationResult({
    required this.tries,
    required this.games,
    required this.wins,
  });

  EvaluationResult.empty({required int playerLength})
      : tries = 0,
        games = 0,
        wins = List.filled(playerLength, 0);

  factory EvaluationResult.fromMatchup(Matchup matchup) {
    MadeHand? bestHand = null;
    List<int> bestPlayerIndexes = [];

    for (int i = 0; i < matchup.playerMadeHands.length; ++i) {
      final madeHand = matchup.playerMadeHands[i];

      if (madeHand.power > (bestHand?.power ?? -1)) {
        bestHand = madeHand;
        bestPlayerIndexes = [i];
      } else if (madeHand.power == bestHand?.power) {
        bestPlayerIndexes.add(i);
      }
    }

    List<int> wins = List.filled(matchup.playerMadeHands.length, 0);
    for (final index in bestPlayerIndexes) {
      wins[index] += 1;
    }

    return EvaluationResult(
      tries: 1,
      games: 1,
      wins: wins,
    );
  }

  /**
   *
   */
  final int tries;
  /**
   *
   */
  final int games;
  /**
   *
   */
  final List<int> wins;

  @override
  String toString() =>
      "EvaluationResult<tries: $tries, games: $games, wins: $wins>";

  EvaluationResult operator +(EvaluationResult other) {
    List<int> winsSum = wins.toList();

    for (int i = 0; i < winsSum.length; ++i) {
      winsSum[i] += other.wins[i];
    }

    return EvaluationResult(
      tries: tries + other.tries,
      games: games + other.games,
      wins: winsSum,
    );
  }
}
